/*
**(c) Copyright 1989,1990,1991 Novell Inc. All Rights Reserved.
*/

/*
** (c) Copyright 1989, 1990 Novell, Inc. All rights reserved.
**
** $Logfile:   G:/glblinc/vcs/csprims.h_v  $
** $Revision:   2.1  $
** $Date:   09 Apr 1992 17:09:38  $
** $Author:   RFROST  $
** $Log:   G:/glblinc/vcs/csprims.h_v  $
** 
**    Rev 2.1   09 Apr 1992 17:09:38   RFROST
** No change.
** 
**    Rev 2.0   07 Apr 1992 13:28:52   KLEUNG
** Beginning of CS 1.3 changes
** 
**    Rev 1.7   07 Apr 1992 11:10:26   KLEUNG
** Add #define for generic send MSID, and primitive ranges for EEAS, DLM.
** 
**    Rev 1.6.1.2   03 Apr 1992 10:32:20   KLEUNG
** Add #define CS_GENERIC_SEND_MSID.
** 
**    Rev 1.6.1.1   29 Mar 1992 10:32:12   RFROST
** Added EEAS base and limit
**
**    Rev 1.6.1.0   24 Feb 1992 11:54:02   RFROST
** OS/2 WCM changes
**
**    Rev 1.6   02 Jul 1991 14:49:14   KLEUNG
** Add PPSA base and limit.
**
**    Rev 1.5   22 May 1991 11:16:08   ARAO
** Added Copyright Notice
**
**    Rev 1.4   11 Sep 1990 07:57:56   MFROST
** Documentation per technical writer.
**
**    Rev 1.3   09 Aug 1990 17:31:24   MFROST
** Added CS_FORCED_DOWN definition.
**
**    Rev 1.2   10 May 1990 22:26:52   MFROST
** Added LCA_BASE definition and deleted CONSOLE_BASE definitiion.
**
**    Rev 1.1   19 Jan 1990 09:32:06   COLSEN
** Updated copyright banner
**
**    Rev 1.0   28 Nov 1989 16:15:48   BBOHDANO
** Initial revision.
*/

#ifndef _CSPRIMS_INCLUDED
#define _CSPRIMS_INCLUDED

#ifndef _CSTYPES_INCLUDED
#include "cstypes.h"
#endif

/*
** For each Executive Module, CSPRIMS.H defines a _BASE and _LIMIT.
** The _BASE and _LIMIT dictate two things:
**   1)  the range of primitive values that the module may support, and
**   2)  the range of error numbers returnable by the module.
**
** Note that in the interface definition file for each module, the _BASE
** is added to the first primitive value and to the first error value.
** Subsequent primitive	values and error values are defined relative to
** the first, so the net effect is that the _BASE is added to each primitive
** and error value.
**
** Novell reserves the space from 0x00 to 0x003FFFFF.
** When creating a new Service Module, contact Novell to be assigned
** _BASE and _LIMIT values in the range 0x00400000 to 0x00FFFFFF.
** Do not modify this file, but write a new #include file to define the
** new assigned _BASE and _LIMIT.
**
** The most significant 8-bit byte (0xFF000000) contains flags that specify
** whether a primitive value is an ack, an error ack, or a priority request.
*/

#define SYSTEM_BASE					0x00000100		/* system generic functions */
#define SYSTEM_LIMIT					0x000001ff

#define CNS_BASE						0x00000300		/* connection service multiplexor */
#define CNS_LIMIT						0x000004ff

#define NMA_BASE						0x00000600		/* network management agent */
#define NMA_LIMIT						0x000007ff

#define LCA_BASE						0x00000900		/* local console agent */
#define LCA_LIMIT						0x000009ff

#define DBA_BASE						0x00000b00		/* database agent */
#define DBA_LIMIT						0x00000bff

#define SNA_BASE						0x00000c00		/* SNA386 agent */
#define SNA_LIMIT						0x00000cff

#define SMA_BASE						0x00000d00		/* service mapping agent */
#define SMA_LIMIT						0x00000dff

#define CM_BASE						0x00000f00		/* connection manager */
#define CM_LIMIT						0x000010ff

#define SSA_BASE						0x00001400		/* security services */
#define SSA_LIMIT						0x000014ff

#define PPSA_BASE						0x00001600		/* peer to peer services */
#define PPSA_LIMIT 					0x000016ff

#define EEAS_BASE						0x00001700		/* access support services */
#define EEAS_LIMIT					0x000017ff

#define DLM_BASE						0x00001800		/* data link manager */
#define DLM_LIMIT						0x000018ff	

/*
** FLAGS: specify whether a primitive is an Ack, an Error Ack, or a
**        Priority request.  A regular request primitive carries no flag.
**        These flags are added to primitive values in the interface
**        definition files.
*/

#define PRIM_ACK						0x80000000		/* bit 31 */
#define PRIM_ERROR					0x40000000		/* bit 30 */
#define PRIM_PRIORITY				0x20000000		/* bit 29 */
																/* bits 28 - 24 reserved */
/*
** STRUCTURE:   CS Header
** DESCRIPTION: identifies to the CNS the source and destination of a
**              primitive.  A block of type CSHdr_t is pointed to by ...
*/

typedef struct {
	U32_t		SrcAgentId;									/* source CS agent ID */
	U32_t		SrcMSId;										/* source MS ID */
	U32_t		DstAgentId;									/* destination CS agent ID */
	U32_t		DstMSId;										/* destination MS ID */
} CSHdr_t;

#define CS_GENERIC_SEND_MSID		0xFFFFFFFF

#ifndef OS2DEF_INCLUDED
/*
**  The following value is placed in the errno variable by the
**  CLIB whenever the CNS multiplexer sends an M_ERROR message to
**  the Stream head.  Upper multiplexed agents should check errno for
**  this value whenever getmsg() returns an error to determine if
**  a Stream head failure has occured (in which case you call CSAbend()) or
**  a forced shutdown is in progress (in which case you simply clean
**  up and close the Stream).
*/
#define CS_FORCED_DOWN				0xff				/* forced shutdown indication */

#endif		/* OS2DEF_INCLUDED */
#endif
