/*
** (c) Copyright 1992 Novell, Inc. All rights reserved.
**
** File: nvcmd.c
**
** $Logfile:   G:/cs/nma/vcs/nvcmd.c_v  $
** $Revision:   1.1  $
** $Date:   04 Nov 1992 17:47:32  $
** $Author:   RWONG  $
** $Log:   G:/cs/nma/vcs/nvcmd.c_v  $
** 
**    Rev 1.1   04 Nov 1992 17:47:32   RWONG
** API requires nlm handles now.
** 
**    Rev 1.1   04 Nov 1992 17:47:02   RWONG
** 
**    Rev 1.0   08 Oct 1992 16:40:04   RWONG
** Initial revision.
*/

/* many of these are included for function prototypes */
#include <library.h>
#include <stdio.h>
#include <string.h>
#include <malloc.h> 
#include <process.h>
#include <signal.h>
#include <conio.h>
#include <errno.h>
#include <sys/types.h>

#include <cstypes.h>
#include <nmvtdef.h>
#include <nvapi.h>

#define MYSCREENNAME "NVCMD Sample Program"
#define APPL_NAME    "NVCMD"

/*  forward declarations  */
int            nvcmd_shutdown (void);     /* NLM sigterm procedure */
void           process_requests (void);
void           callbackfunc (u32_t, void *);

/*  global variables  */
u32_t          testNLMHandle;               /* nlm handle */ 
int            testThreadGrpId = 0;         /* main thread group ID */
int            testMainThreadId = 0;        /* main thread ID */
int            screenhdl;                   /* handle to i/o screen */

/*  various states of this NLM  */
#define        TEST_INIT          0x00      /* initial state */
#define        TEST_SIGNAL        0x10      /* signal main thread to cleanup */
#define        TEST_UNLOAD        0x20      /* ok to unload NLM */
u32_t          testStatus = TEST_INIT;      /* my current status */

/*  structure for the queue we create for run command requests */
typedef struct _runcmdq {
struct _runcmdq   *next;
NVNMVT_t          request;
} RunCmdQueue_t;

RunCmdQueue_t  *QueueHead = NULL;
RunCmdQueue_t  *QueueTail = NULL;

#define MSG1 "This demonstrates how to send a RunCmd response"
#define MSG2 "with a multiple line format.  This allows more"
#define MSG3 "control of the presentation on the NetView console."
#define NUM_OF_LINES 3

NVNMVT_t *multirsp;


void main (void)
{
   u32_t    rc;
   size_t   len;

   if ((screenhdl = CreateScreen (MYSCREENNAME, 0)) == EFAILURE)
      exit (0);

   /*  
   **  obtain NLM handle, thread group ID and main thread ID 
   */
   testNLMHandle = GetNLMHandle ();
   testThreadGrpId = GetThreadGroupID ();
   testMainThreadId = GetThreadID ();

   /* registers SIGTERM handler  */
   if (signal (SIGTERM, nvcmd_shutdown) == SIG_ERR)
      exit (0);
  
   /* tell NetView agent we want to be managed */
   rc = NWNVRegister (APPL_NAME, testNLMHandle, (void *)callbackfunc);
   printf ("NWNVRegister returned %x\r\n", rc);

   /*
   ** Format the "canned" response message.  This message will be
   ** three lines.  Since the NVNMVT_t structure has space for only
   ** the first line's NVMsg_t structure, allocate space for two
   ** more.
   */
   len = sizeof (NVNMVT_t) + (NUM_OF_LINES - 1) * sizeof (NVMsg_t);

   multirsp = (NVNMVT_t *)__qcalloc (1, len);
   multirsp->MsgCount = 3;
   multirsp->Msg[0].MsgLen = strlen (MSG1);
   multirsp->Msg[0].MsgData = MSG1;
   multirsp->Msg[1].MsgLen = strlen (MSG2);
   multirsp->Msg[1].MsgData = MSG2;
   multirsp->Msg[2].MsgLen = strlen (MSG3);
   multirsp->Msg[2].MsgData = MSG3;

   while (1) {
      /* wait for work */
      SuspendThread (testMainThreadId);

      /* process all requests currently queued */
      process_requests ();

      /* check if we're being unloaded */
      if (testStatus & TEST_SIGNAL)
         break;
   }   

   /* 
   ** Flag has been set indicating we're being unloaded.
   ** Inform NetView agent that we're going away.
   */
   rc = NWNVDeregister (testNLMHandle);
   printf ("NWNVDeregister returned %x\r\n", rc);

   /* free canned response message */
   free ((char *)multirsp);
     
   testStatus |= TEST_UNLOAD;         /*  signal SIGTERM handler that it is OK to unload  */
   ExitThread (EXIT_THREAD, 0);       /*  terminate main thread  */

}



void process_requests (void)
{
   u32_t          rc;
   RunCmdQueue_t  *qptr;

   /* while there are still requests on the queue */
   while ((qptr = QueueHead) != (RunCmdQueue_t *)NULL) {
      QueueHead = QueueHead->next;

      if (QueueHead == (RunCmdQueue_t *)NULL)
         QueueTail = (RunCmdQueue_t *)NULL;

      printf ("command is %s\r\n", qptr->request.Msg[0].MsgData);

      if (!(testStatus & TEST_SIGNAL)) {
         printf ("processing command\r\n");

         multirsp->RequestId = qptr->request.RequestId;
         rc = NWNVSend (testNLMHandle, NVTYPE_RUNCMD_RSP, (void *)multirsp);

         printf ("NWNVSend returned %x\r\n", rc);
      }
      else {
         printf ("discarding command - unload in progress\r\n");
      }
      
      free (qptr);
   }
}


int nvcmd_shutdown()
{
   signal (SIGTERM, SIG_IGN);

   /*  if main thread is suspended, resume it to perform cleanup  */
   ResumeThread (testMainThreadId);
   testStatus |= TEST_SIGNAL;

   /*  wait for cleanup to be completed by main thread - TEST_UNLOAD bit would be set then  */
   while (!(testStatus & TEST_UNLOAD))
      ThreadSwitch ();

   return (NULL);

}



void callbackfunc (u32_t isruncmd, void *data)
{
   u32_t          currentThreadGrpId;
   RunCmdQueue_t  *qptr;
   NVNMVT_t       *RunCmd;

   /* make sure printf goes to our screen */
   (void)SetCurrentScreen (screenhdl);

   /* if this is not a runcmd, it is an error indication */
   if (!isruncmd) {
      printf ("\r\nerror %x received\r\n", ((NVErrData_t*)data)->Error);
      return;
   }

   /* make a private copy of the request */                                                                         
   RunCmd = (NVNMVT_t *)data;

   /*
   ** We need to set the thread group before allocating memory or NetWare
   ** will think that the NetView agent owns it.
   */
   currentThreadGrpId = SetThreadGroupID (testThreadGrpId);
   qptr = (RunCmdQueue_t *)__qcalloc (1, sizeof (RunCmdQueue_t) + RunCmd->Msg[0].MsgLen + 1);
   SetThreadGroupID (currentThreadGrpId);

   if (qptr != (RunCmdQueue_t *)NULL) {
      qptr->request.RequestId = RunCmd->RequestId;
      qptr->request.MsgCount = RunCmd->MsgCount;      // will always be 1
      qptr->request.Msg[0].MsgLen = RunCmd->Msg[0].MsgLen;

      /*
      ** Copy the text of the run command request to the end of the
      ** buffer we just allocated.  The run command request will always
      ** be received as one line.
      */ 
      qptr->request.Msg[0].MsgData = (u8_t *)qptr + sizeof (RunCmdQueue_t);
      memcpy (qptr->request.Msg[0].MsgData, RunCmd->Msg[0].MsgData, RunCmd->Msg[0].MsgLen);      

      /*
      ** Add EOS character, since in this sample we will print the
      ** run command on the screen.  Note that we allocated an extra
      ** byte for qptr in order to do this.
      */
      qptr->request.Msg[0].MsgData[RunCmd->Msg[0].MsgLen] = '\0';
   }

   /*
   ** Enqueue the request.  If there is nothing currently on
   ** the queue, resume the our main thread so that the new
   ** request will be processed.  If there are already requests
   ** on the queue, add the new request to the end.
   */
   if (QueueTail == (RunCmdQueue_t *)NULL) {
      QueueTail = QueueHead = qptr;
      ResumeThread (testMainThreadId);
   }
   else {
      QueueTail->next = qptr;
      QueueTail = qptr;
   }
}
