/*
**  (C) Copyright 1992 Novell, Inc.  All rights reserved.
**
**  File: nvipx.c
**
**  Description: This demonstrates how to send an alert to a host using
**  the Open NetView IPX socket based API.  This code is for an NLM.
**  See the sample program NVIPXWS.C if coding an EXE.
**
**  $Logfile:   J:/cs/nma/vcs/nvipx.c_v  $
**  $Revision:   1.3  $
**  $Date:   14 Apr 1993 16:20:44  $
**  $Author:   RWONG  $
**  $Log:   J:/cs/nma/vcs/nvipx.c_v  $
** 
**    Rev 1.3   14 Apr 1993 16:20:44   RWONG
** Change adapter type to coded graphic character set.
** 
**    Rev 1.2   03 Nov 1992 18:28:34   RWONG
** Change alert to open failure - token ring lobe.
** 
**    Rev 1.1   13 Oct 1992 12:19:06   RWONG
** Changed header description.
** 
**    Rev 1.0   12 Oct 1992 14:47:40   RWONG
** Initial revision.
*/

/* many of these are included for function prototypes */
#include <stdlib.h>
#include <stdio.h>
#include <process.h>
#include <string.h>
#include <sys\types.h>
#include <nwipxspx.h>
#include <nwbindry.h>
#include <nwenvrn.h>

#include <cstypes.h>
#include <nmvtdef.h>
#include <nvapi.h>
#include <nvipx.h>

/* forward reference */
void     send_alert(void);

/*
** This is the msg data for a fully formatted alert NMVT.
** This is a OPEN FAILURE alert for a TOKEN RING LOBE
**
** Generic Alert Description Code Point
**    3211 - open failure
**
** Probable Causes
**    3702 - token ring lobe
**
** Failure Causes subfields
**    3320, 3711, 3434, F8E0, 33A5
**
** Recommended Action subfields
**    1009, 3301, 2010, 3101, 32C0, 31E0
**
** LAN link connection subsystem data subvector
** supporting data correlation subfield
**    03 - Local individual Mac Address
*/

#define NMVTDATALEN  151

u8_t  msg[NMVTDATALEN] = {
0x41,0x03,0x8d,0x00,0x00,0x00,0x00,0x00,0x00,0x8f,0x00,0x00,
0x23,0x10,0x00,0x20,0x11,0x03,0x13,0x00,0x11,0xF0,0xF3,0xF8,
0xF6,0x81,0x81,0x81,0x89,0x82,0xF3,0xF4,0xF5,0xF6,0xF4,0xF6,
0xF7,0x0A,0x0E,0xC9,0xC2,0xD4,0xE3,0xD6,0xD2,0xC5,0xD5,
0x0b,0x92,0x00,0x00,0x01,0x32,0x11,0x00,0x00,0x00,0x00,0x04,
0x93,0x37,0x02,0x47,0x96,0x0C,0x01,0x33,0x20,0x37,0x11,  
0x34,0x34,0xF8,0xE0,0x33,0xA5,0x03,0x83,0x21,0x19,0x82,0x00,
0x35,0x11,0xC9,0xC2,0xD4,0x40,0xE3,0xD6,0xD2,0xC5,0xD5,0x40,
0xD9,0xC9,0xD5,0xC7,0x40,0xC2,0xD6,0xC1,0xD9,0xC4,0x0E,0x81,  
0x10,0x09,0x33,0x01,0x20,0x10,0x31,0x01,0x32,0xC0,0x31,0xE0,
0x06,0x82,0x91,0x61,0x01,0xFF,0x06,0x82,0x01,0x07,0x00,0xFF,
0x03,0x83,0x21,0x0A,0x51,0x08,0x03,0x01,0x02,0x03,0x04,0x05,  
0x06,0x08,0x01,0x5C,0x09,0x1B,0x0F,0x05,0x2A};

/* Buffer for name of the server generating the alert */
char     server_name[49];

/* Buffer for name of the destination server */
char     target_server[49];

/* ECB for the outgoing alert message */
IPX_ECB  ecbBuf;

/* IPX address of the sending station */
struct Source_s {
char     network[4];
char     node[6];
} source;

/*
*  IPX address of the destination.  The reserved field is necessary
*  because addr is obtained using the bindery API call FindPropertyValue
*  which returns 128 bytes.
*/
struct Dest_s {
char              network[4];
char              node[6];
unsigned short    socket;
char              reserved[116];    /* ReadPropertyValue can return 128 bytes */
} dest;

/* Buffer for the alert and the IPX header */
typedef struct {
IPX_HEADER     hdr;
unsigned char  ipxData[1];
} EventECB;
 

void main (void)
{
   unsigned char flag = 0, more = 0;

   /* get the source network/node address and local server name */
   (void) IpxGetInternetworkAddress((unsigned char *) &source);
   (void) GetFileServerName(0, server_name);  /* get local server name */

   printf("\r\nEnter target server name: ");
   gets(target_server);

   /* convert to upper case for bindery comparison */
   strupr(target_server);

   /* find the CE in the bindery with the given target name */
   if (ReadPropertyValue(target_server, 0x0130, "NET_ADDRESS", 1,
                         (unsigned char *) &dest, &more, &flag) != 0)
   {
      printf("\r\nUnknown server or server not running NetWare for SAA.\r\n");
      exit (0);
   }

   send_alert ();

   ExitThread (EXIT_THREAD, 0);
}


void send_alert()
{
   u32_t             size, transport;
   EventECB          *databuf;
   NMVTEvt_t         *eventptr;

   memset((char *) &ecbBuf, '\0', sizeof(IPX_ECB));

   /* fetch the immediate address into the ECB */
   dest.socket = NVSOCKET;
   (void) IpxGetLocalTarget((unsigned char *) &dest, &ecbBuf, &transport);

   size = sizeof (IPX_HEADER)            
            + sizeof(NMVTEvt_t) - 1 
            + NMVTDATALEN;                /* actual alert data len */

   if ((databuf = (EventECB *)calloc(1, size)) == NULL) 
      return;

   eventptr = (NMVTEvt_t *)databuf->ipxData;

   eventptr->PrimType = NM_EVENT_NOTIFICATION;   
   eventptr->InputType = NVEVENT_NMVT;
   (void)strcpy ((char *)eventptr->ServerName, server_name);

   eventptr->EventDataCount = 1;          /* always 1 data vector */

   eventptr->EventData.DataType = NMVT_TYPE;       /* data vector type */
   eventptr->EventData.Count = 1;                  /* always 1 */
   eventptr->EventData.DataLen = NMVTDATALEN;

   (void)memcpy (eventptr->EventData.NMVTData, msg, NMVTDATALEN);

   /* finish filling in the ECB */   
   ecbBuf.fragList[0].fragAddress = (void *)databuf;
   ecbBuf.fragCount = 1;
   ecbBuf.fragList[0].fragSize = size;
   ecbBuf.socket = 0;

   /* fill in the IPX header */
   databuf->hdr.packetType = 4;           /* IPX */

   memcpy((char *) &databuf->hdr.destNet, dest.network, 4); 
   memcpy(databuf->hdr.destNode, dest.node, 6);
   databuf->hdr.destSocket = NVSOCKET;

   memcpy((char *) &databuf->hdr.sourceNet, source.network, 4);
   memcpy(databuf->hdr.sourceNode, source.node, 6);
   databuf->hdr.sourceSocket = 0;

   /* send the alert to the CE */
   IpxSend(0, &ecbBuf);

   free((void *) databuf);
}      


